@@ -1,8 +1,9 @@
-# Version 1.1-1
+# Version 1.2-1
 
 import re
 import sys
 from hostlist import expand_hostlist, collect_hostlist
+import commands
 
 class NodesException(Exception):
     def __init__(self, msg):
@@ -14,6 +15,7 @@ class MissingGroup(NodesException): pass
 class DuplicateMembership(NodesException): pass
 class UnknownLevel(NodesException): pass
 class AmbiguousLevel(NodesException): pass
+class FailedSLURM(NodesException): pass
 
 class Entity:
     pass
@@ -75,12 +77,19 @@ class Hierarchy:
 
     def create_group(self, level, name, all_nodes_group = False):
         #print "GROUP", level, name
+        self.level_names.add(level)
         if name in self.names:
             raise DuplicatedGroup("group %s seen twice" % name)
         group = Group(self, level, name, all_nodes_group)
         self.names[name] = group
         return group
 
+    def create_or_get_group(self, level, name):
+        if name in self.names:
+            return self.names[name]
+        else:
+            return self.create_group(level, name)
+
     def get_entity(self, name, auto_add_node = False):
         if name not in self.names:
             if auto_add_node:
@@ -111,7 +120,6 @@ class Hierarchy:
             if m: 
                 (level, name, rest) = m.group(1,2,3)
                 current_group = self.create_group(level, name)
-                self.level_names.add(level)
                 if rest:
                     current_group.add_nodes(rest)
                 continue
@@ -127,6 +135,23 @@ class Hierarchy:
             # Fail
             raise BadConfigSyntax(line)
 
+    def parse_slurm(self):
+        rc, slurm_data = commands.getstatusoutput('/usr/bin/squeue -aho "%i %u %N"')
+        if rc <> 0:
+            raise FailedSLURM("Failed to get data from SLURM")
+
+        for line in slurm_data.split("\n"):
+            fields = line.split()
+            if len(fields) <> 3:
+                continue
+            job, user, nodes = fields
+
+            gj = self.create_group("job", "job" + job)
+            gj.add_nodes(nodes)
+
+            gu = self.create_or_get_group("user", user)
+            gu.add_nodes(nodes)
+
     def expand_abbreviated_level(self, level):
         matching = [l for l in self.level_names if l.startswith(level)]
         if len(matching) == 1:
